@@ -108,15 +108,15 @@ impl TransportMulticast {
         }
     }
 
+    /// Returns if the message has successfully been sent.
     #[inline(always)]
-    pub fn schedule(&self, message: NetworkMessageMut) -> ZResult<()> {
+    pub fn schedule(&self, message: NetworkMessageMut) -> ZResult<bool> {
         let transport = self.get_transport()?;
-        transport.schedule(message)?;
-        Ok(())
+        transport.schedule(message)
     }
 
     #[cfg(feature = "stats")]
-    pub fn get_stats(&self) -> ZResult<Arc<common::stats::TransportStats>> {
+    pub fn get_stats(&self) -> ZResult<zenoh_stats::TransportStats> {
         Ok(self.get_transport()?.stats.clone())
     }
 }
