@@ -16,11 +16,13 @@ use std::time::Duration;
 use tokio_util::{sync::CancellationToken, task::TaskTracker};
 use zenoh_buffers::ZSliceBuffer;
 use zenoh_link::Link;
+#[cfg(feature = "unstable")]
+use zenoh_protocol::core::Priority;
 use zenoh_protocol::transport::{KeepAlive, TransportMessage};
 use zenoh_result::{zerror, ZResult};
+#[cfg(feature = "unstable")]
+use zenoh_sync::{event, Notifier, Waiter};
 use zenoh_sync::{RecyclingObject, RecyclingObjectPool};
-#[cfg(feature = "stats")]
-use {crate::common::stats::TransportStats, std::sync::Arc};
 
 use super::transport::TransportUnicastUniversal;
 use crate::{
@@ -44,8 +46,15 @@ pub(super) struct TransportLinkUnicastUniversal {
     // The task handling substruct
     tracker: TaskTracker,
     token: CancellationToken,
+    #[cfg(feature = "unstable")]
+    // Notifier for a BlockFirst message to be ready to be sent
+    // (after the previous one has been sent)
+    pub block_first_notifiers: [Notifier; Priority::NUM],
+    #[cfg(feature = "unstable")]
+    // Waiter for a BlockFirst message to be ready to be sent
+    pub block_first_waiters: [Waiter; Priority::NUM],
     #[cfg(feature = "stats")]
-    pub(super) stats: Arc<TransportStats>,
+    pub(super) stats: zenoh_stats::LinkStats,
 }
 
 impl TransportLinkUnicastUniversal {
@@ -75,13 +84,35 @@ impl TransportLinkUnicastUniversal {
         // The pipeline
         let (producer, consumer) = TransmissionPipeline::make(config, priority_tx);
 
+        #[cfg(feature = "stats")]
+        let stats = transport
+            .stats
+            .link_stats(link.link.get_src(), link.link.get_dst());
+
+        #[cfg(feature = "unstable")]
+        let mut block_first_notifiers = Vec::new();
+        #[cfg(feature = "unstable")]
+        let mut block_first_waiters = Vec::new();
+        #[cfg(feature = "unstable")]
+        for _ in 0..Priority::NUM {
+            let (notifier, waiter) = event::new();
+            // notify to be make the BlockFirst "slot" available
+            notifier.notify().unwrap();
+            block_first_notifiers.push(notifier);
+            block_first_waiters.push(waiter);
+        }
+
         let result = Self {
             link,
             pipeline: producer,
             tracker: TaskTracker::new(),
             token: CancellationToken::new(),
+            #[cfg(feature = "unstable")]
+            block_first_notifiers: block_first_notifiers.try_into().ok().unwrap(),
+            #[cfg(feature = "unstable")]
+            block_first_waiters: block_first_waiters.try_into().ok().unwrap(),
             #[cfg(feature = "stats")]
-            stats: TransportStats::new(Some(Arc::downgrade(&transport.stats)), Default::default()),
+            stats,
         };
 
         (result, consumer)
@@ -188,7 +219,7 @@ async fn tx_task(
     link: &mut TransportLinkUnicastTx,
     keep_alive: Duration,
     token: CancellationToken,
-    #[cfg(feature = "stats")] stats: Arc<TransportStats>,
+    #[cfg(feature = "stats")] stats: zenoh_stats::LinkStats,
 ) -> ZResult<()> {
     loop {
         tokio::select! {
@@ -199,8 +230,8 @@ async fn tx_task(
 
                         #[cfg(feature = "stats")]
                         {
-                            stats.inc_tx_t_msgs(batch.stats.t_msgs);
-                            stats.inc_tx_bytes(batch.len() as usize);
+                            stats.inc_bytes(zenoh_stats::Tx, batch.len() as u64);
+                            stats.inc_transport_message(zenoh_stats::Tx, batch.stats.t_msgs as u64);
                         }
 
                         // Reinsert the batch into the queue
@@ -220,8 +251,8 @@ async fn tx_task(
 
                         #[cfg(feature = "stats")]
                         {
-                            stats.inc_tx_t_msgs(1);
-                            stats.inc_tx_bytes(n);
+                            stats.inc_bytes(zenoh_stats::Tx, n as u64);
+                            stats.inc_transport_message(zenoh_stats::Tx, 1);
                         }
                     }
                 }
@@ -240,8 +271,8 @@ async fn tx_task(
 
         #[cfg(feature = "stats")]
         {
-            stats.inc_tx_t_msgs(b.stats.t_msgs);
-            stats.inc_tx_bytes(b.len() as usize);
+            stats.inc_bytes(zenoh_stats::Tx, b.len() as u64);
+            stats.inc_transport_message(zenoh_stats::Tx, b.stats.t_msgs as u64);
         }
     }
 
@@ -254,7 +285,7 @@ async fn rx_task(
     lease: Duration,
     rx_buffer_size: usize,
     token: CancellationToken,
-    #[cfg(feature = "stats")] stats: Arc<TransportStats>,
+    #[cfg(feature = "stats")] stats: zenoh_stats::LinkStats,
 ) -> ZResult<()> {
     async fn read<T, F>(
         link: &mut TransportLinkUnicastRx,
@@ -292,7 +323,8 @@ async fn rx_task(
                 let batch = batch.map_err(|_| zerror!("{}: expired after {} milliseconds", link, lease.as_millis()))??;
                 #[cfg(feature = "stats")]
                 {
-                    stats.inc_rx_bytes(2 + batch.len()); // Account for the batch len encoding (16 bits)
+                    let header_bytes = if l.is_streamed { 2 } else { 0 };
+                    stats.inc_bytes(zenoh_stats::Rx, header_bytes + batch.len() as u64);
                 }
                 transport.read_messages(batch, &l, #[cfg(feature = "stats")] &stats)?;
             }
